// ====================================================================
// Copyright (C) 2011 - Tsubasa Saito
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
// ====================================================================

// *****************************************************************
// N-th rootof the DD number
//
// Calling Sequence :
// ddnrt(a,n)
//
// Arguments :
// a : DD number (limited DD type value size 1 by 1)
// n : integer
//
// Authors :
// Copyright (C) 2011 - Tsubasa Saito
// last updating :
// *****************************************************************
function sq = ddnrt(a,N)
    if prod(size(a.hi))~=1 then
        msg = _("%s: Argument #%d: Scalar (1 element) expected.\n")
        error(msprintf(msg, "ddnrt", 1));
    else
        if(N <= 0) then
            msg = _("%s: Argument #%d: Must be > %d.\n")
            error(msprintf(msg, "ddnrt", 2, 0));
        end
        if(a.hi > 0) then
            c = a.hi^(1.0/N); // Initial value

            p = c*c;
            t = 134217729 * c;
            ah = t - (t - c);
            al = c - ah;
            e = ((ah * ah - p) + 2 * ah * al) + al * al;

            pp = p * c;
            t = 134217729 * p;
            ah = t - (t - p);
            al = p - ah;
            t = 134217729 * c;
            bh = t - (t - c);
            bl = c - bh;
            ee = ((ah * bh - pp) + ah * bl + al * bh) + al * bl;
            ee = ee + e * c;
            tmp = pp + ee;
            ee = ee - (tmp - pp);
            pp = tmp;

            for i = 3:(N-1)
                p = pp;
                pp = p * c;
                t = 134217729 * p;
                ah = t - (t - p);
                al = p - ah;
                t = 134217729 * c;
                bh = t - (t - c);
                bl = c - bh;
                tmp = ((ah * bh - pp) + ah * bl + al * bh) + al * bl;
                ee = tme + ee * c;
                tmp = pp + ee;
                ee = ee - (tmp - pp)
                pp = tmp
            end
            cc = (a.hi - pp - ee + a.lo) / N; // Newton Iterative
            cc = cc/p;

            tmp =  c + cc;
            sq = tlist(['dd','hi','lo'],tmp,cc-(tmp-c));
        elseif(hi == 0 & lo == 0)
            sq = tlist(['dd','hi','lo'],0,0);
        else
            msg = _("%s: Argument #%d: Must be > %d.\n")
            error(msprintf(msg, "ddnrt", 1, 0));
        end
    end
endfunction

