// ====================================================================
// Copyright (C) 2011 - Tsubasa Saito
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
// ====================================================================

// *****************************************************************
// Quick renormalize function
// This is an internal function
//
// Calling Sequence :
// renormalize(a1,a2,a3,a4,a5)
//
// This function is based on Bailey's QD
// http://crd.lbl.gov/~dhbailey/mpdist/
//
// Arguments :
// a1,a2,a3,a4,a5 : double numbers
//
// Authors :
// Copyright (C) 2011 - Tsubasa Saito
// last updating : 
// *****************************************************************
function [b1,b2,b3,b4] = renormalize(a1,a2,a3,a4,a5);
    //[s,t4] = fast_two_sum(a4,a5);
    s = a4 + a5;
    t4 = a5 - (s - a4);
    //[ss,t3] = fast_two_sum(a3,s);
    ss = a3 + s;
    t3 = s - (ss - a3);
    //[s,t2] = fast_two_sum(a2,ss);
    s  = a2 + ss;
    t2 = ss - (s - a2);
    //[b1,t1] = fast_two_sum(a1,s);
    b1 = a1 + s;
    t1 = s - (b1 - a1);
    //[s,t3] = fast_two_sum(t3,t4);
    s = t3 + t4;
    t3 = t4 - (s - t3);
    //[ss,t2] = fast_two_sum(t2,s);
    ss = t2 + s;
    t2 = s - (ss - t2);
    //[b2,t1] = fast_two_sum(t1,ss);
    b2 = t1 + ss;
    t1 = ss - (b2 - t1);
    //[s,t2] = fast_two_sum(t2,t3);
    s = t2 + t3;
    t2 = t3 - (s -t2);
    //[b3,t1] = fast_two_sum(t1,s);
    b3 = t1 + s;
    t1 = s - (b3 - t1);
    b4 = t1 + t2;
endfunction
