// ====================================================================
// Copyright (C) 2011 - Tsubasa Saito
// Copyright (C) 2018 - Samuel GOUGEON
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
// ====================================================================

// *****************************************************************
// QR decomposition with Householder
//
// Calling Sequence :
// qr(A)
//
// Arguments :
// A : DD or QD matrix
//
// Return :
// Q : (m,m) orthogonal matrix
// R : (m,n) upper triangular matrix
//
// Authors :
// Copyright (C) 2011 - Tsubasa Saito
// Copyright (C) 2018 - Samuel GOUGEON
// last updating : 2018-07-31
// *****************************************************************

// Internal called by %dd_qr() and %qd_qr()
function [Q,R] = ddqd_qr(A)
    [m,n] = size(A(2));
    Q = eye(A)
    R = A;
    z = zeros(A(:,1))
    for k = 0:min(m,n)-1
        v = z(1:m-k,1);
        v(1:m-k,1) = R(k+1:m,k+1);
        a = (-1.0*sign(v(2)(1)))* norm(v,2);
        v(1,1) = v(1,1) - a;
        tmp = 2.0/(v'*v)
        R(k+1:m,k+1:n) = R(k+1:m,k+1:n) - tmp*(v*(v'*R(k+1:m,k+1:n)));
        Q(k+1:m,:) = Q(k+1:m,:) - tmp*(v*(v'*Q(k+1:m,:)));
    end
    Q = Q';
endfunction

