// ====================================================================
// Copyright (C) 2011 - Tsubasa Saito
// Copyright (C) 2018 - Samuel GOUGEON
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
// ====================================================================

// *****************************************************************
// Input DD number
//
// Calling Sequence :
// str2dd(a)
//
// Arguments :
// a : string
//
// This function is based on Bailey's DDFUN90
// http://crd.lbl.gov/~dhbailey/mpdist/
//
// Authors :
// Copyright (C) 2011 - Tsubasa Saito
// Copyright (C) 2018 - Samuel GOUGEON
// last updating : 2018-07-31
// *****************************************************************
function b = str2dd(a)

    digits = '0123456789';
    id = 0;
    ip = -1;
    is = 0;
    inz = 0;
    i_exp = [];     // position of the e|E|d|D symbol, if any
    f = tlist(['dd','hi','lo'],0,0);
    s = tlist(['dd','hi','lo'],0,0);

    c = strsplit(stripblanks(a));

    inn = length(a);

    // scan for digits, looking for the period also.

    for i = 1:inn
        ci = c(i);
        if (ci == '.') then
            if (ip >= 0)
                break;
            end
            ip = id;
            inz = 1;
        elseif (ci == '+') then
            if (id ~= 0 | ip >= 0 | is ~= 0)
                break;
            end
            is = 1;
        elseif(ci == '-') then
            if(id ~= 0 | ip >= 0 | is ~= 0)
                break;
            end
            is = -1;
        elseif (ci == 'e' | ci == 'E' | ci == 'd' | ci == 'D') then
            i_exp = i;
            break;
        elseif (mtlb_findstr(ci,digits) == []) then
            break;
        else
            bi = mtlb_findstr(ci,digits) - 1;
            if(inz > 0 | bi > 0) then
                inz = 1;
                id = id + 1;
                s = 10 * s;
                f.hi = bi;
                s = s + f;
            end
        end
    end
    if (is == -1) then
        s.hi = -s.hi;
        s.lo = -s.lo;
    end

    k=i;

    if (ip == -1) then
        ip = id;
    end

    ie = 1;
    is = 0;
    ca = '';


    for i = k+1:inn
        ci = c(i);
        if (ci == ' ') then
        elseif (ci == '+') then
            if (ie ~= 0 | is ~= 0)
                break
            end
            is = 1;
        elseif (ci == '-') then
            if (ie ~= 0 | is ~= 0)
                break
            end
            is = -1;
        elseif (mtlb_findstr(ci,digits) == []) then
                break
        else
            ie = ie + 1;
            if (ie >= 3) then
                break
            end
            ca(ie) = ci;
        end
    end

    ie = 0; 
    for i = 1:length(strcat(ca))
        k = mtlb_findstr(ca,digits) - 1;
        if (k < 0) then
            break
        elseif (k <= 9) then
            ie = 10 * ie + k;
        end
    end

    if (is == -1) then
        ie = -ie;
    end
    ie = ie + ip - id;

    // Exponent
    e = 0;
    if i_exp~=[] then
        e = strtod(strcat(c(i_exp+1:$)))
        if ~isnan(e)
            e = fix(e)
        else
            e = 0
        end
    end
    s2 = dd(10,0);
    b = s*ddpow(s2,ie(1)+e);
endfunction

